package com.gic.haoban.manage.service.service.out.impl;

import com.alibaba.fastjson.JSONObject;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.commons.util.ToolUtil;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.dto.StoreGroupDTO;
import com.gic.enterprise.api.service.StoreGroupService;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.base.api.common.Constant;
import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.manage.api.dto.DealParamMqDTO;
import com.gic.haoban.manage.api.dto.DepartmentDTO;
import com.gic.haoban.manage.api.dto.SyncCheckDTO;
import com.gic.haoban.manage.api.dto.SyncSingleDealDTO;
import com.gic.haoban.manage.api.enums.PreDealStatusEnum;
import com.gic.haoban.manage.api.enums.PreDealTypeEnum;
import com.gic.haoban.manage.api.enums.SyncTaskStatusEnum;
import com.gic.haoban.manage.api.service.DealSyncOperationApiService;
import com.gic.haoban.manage.api.service.DepartmentApiService;
import com.gic.haoban.manage.service.entity.TabHaobanDepartment;
import com.gic.haoban.manage.service.entity.TabHaobanPreDealLog;
import com.gic.haoban.manage.service.entity.TabHaobanSyncTask;
import com.gic.haoban.manage.service.service.DepartmentService;
import com.gic.haoban.manage.service.service.PreDealService;
import com.gic.haoban.manage.service.service.SyncTaskService;
import com.gic.haoban.manage.service.task.BaseSyncOperation;
import com.gic.mq.sdk.GicMQClient;
import com.gic.redis.data.util.RedisUtil;
import com.task.allocation.api.AbstractTaskAllocationNewOperation;
import com.task.allocation.exception.TaskAllocationException;
import com.task.allocation.qo.TaskAllocationComputed;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * Created 2020/4/9.
 *
 * @author hua
 */
@Service
public class DealSyncOperationApiServiceImpl implements DealSyncOperationApiService {

    private static final Logger logger = LoggerFactory.getLogger(DealSyncOperationApiServiceImpl.class);

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private StoreGroupService storeGroupService;

    @Autowired
    private PreDealService preDealService;

    @Autowired
    private DepartmentApiService departmentApiService;

    @Autowired
    private SyncTaskService syncTaskService;

    @Autowired
    private StoreService storeService;

    @Autowired
    private Map<String, BaseSyncOperation> operationMap;

    private static final String LOCK_KEY = "sync_wx_lock_";

    private static final String LOCK_SUCCESS_KEY = "sync_wx_success_lock_";

    @Override
    public void departmentDealMq(String params) {
        logger.info("mq处理中:{}", params);
        DealParamMqDTO dealParamMqDTO = JSONObject.parseObject(params, DealParamMqDTO.class);
        TabHaobanPreDealLog dataPre = preDealService.getByDataId(dealParamMqDTO.getTaskId()
                , dealParamMqDTO.getData(), PreDealStatusEnum.all.getVal());

        if (dataPre == null) {
            logger.info("待处理数据不存在:{}", params);
            return;
        }
        //部门处理
        if (dataPre.getDataType().equals(PreDealTypeEnum.group.getVal())) {
            operationMap.get("groupSyncOperation").dealSingleByMq(dealParamMqDTO, dataPre);
            //门店处理
        } else if (dataPre.getDataType().equals(PreDealTypeEnum.store.getVal())) {
            logger.info("门店处理：{}", params);
            operationMap.get("storeSyncOperation").dealSingleByMq(dealParamMqDTO, dataPre);

            //成员处理
        } else if (dataPre.getDataType().equals(PreDealTypeEnum.clerk.getVal())) {
            logger.info("店员处理：{}", params);
            operationMap.get("clerkSyncOperation").dealSingleByMq(dealParamMqDTO, dataPre);
        } else {
            logger.info("不是正常数据:{}", params);
        }

    }


    @Override
    public void dealDepartment(String taskId, String wxEnterpriseId
            , List<SyncSingleDealDTO> addList, List<SyncSingleDealDTO> editList) {

        logger.info("初始开始:{}", taskId);
        //处理修改
        dealEidtSync(editList);

        if (CollectionUtils.isEmpty(addList)) {
            syncTaskService.updateTaskStatus(taskId, SyncTaskStatusEnum.compute.getVal());
            unlockTask(wxEnterpriseId);
            return;
        }
        logger.info("处理分组信息开始：{}", taskId);
        dealAddSync(taskId, wxEnterpriseId, addList);

        //加入队列处理
        logger.info("初始化到队列：{}", taskId);
        dealDepartmentInit(taskId, wxEnterpriseId, addList);

    }

    @Override
    public void reDealDepartment(String taskId, int dataType) {
        TabHaobanSyncTask task = syncTaskService.getSyncTask(taskId);
        if (null == task) {
            logger.info("任务不存在!{}", taskId);
            return;
        }
        if ((!task.getStatusFlag().equals(SyncTaskStatusEnum.exception_close.getVal())) &&
                (!task.getStatusFlag().equals(SyncTaskStatusEnum.exception_compute.getVal()))) {
            logger.info("任务不在没有在异常状态!{}", taskId);
            return;
        }
        int groupErrCount = preDealService.countExcepAndPreByTaskId(taskId, PreDealTypeEnum.group.getVal());
        int storeErrCount = preDealService.countExcepAndPreByTaskId(taskId, PreDealTypeEnum.store.getVal());
        List<TabHaobanPreDealLog> list = null;
        if (dataType == -1) {
            if (groupErrCount > 0) {
                dataType = PreDealTypeEnum.group.getVal();
                list = preDealService.queryDepartRebuildDealLog(taskId);
            } else if (storeErrCount > 0) {
                dataType = PreDealTypeEnum.store.getVal();
                list = preDealService.listRebuildByTaskId(taskId, dataType);
            } else {
                dataType = PreDealTypeEnum.clerk.getVal();
                list = preDealService.listRebuildByTaskId(taskId, dataType);
            }
        }
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        Set<String> midList = list.stream().map(TabHaobanPreDealLog::getDataId).collect(Collectors.toSet());
        //根据类型获取状态
        SyncTaskStatusEnum syncTaskStatusEnum = (dataType == PreDealTypeEnum.group.getVal() ? SyncTaskStatusEnum.group_sync
                : (dataType == PreDealTypeEnum.store.getVal() ? SyncTaskStatusEnum.store_sync : SyncTaskStatusEnum.clerk_sync));
        dealDepartmentToMq(taskId, midList, syncTaskStatusEnum);
    }

    @Override
    public String createTask(String wxEnterpriseId, String taskName, String userId, String desc) {
        TabHaobanSyncTask tabHaobanSyncTask = new TabHaobanSyncTask();
        tabHaobanSyncTask.setAddUser(userId);
        tabHaobanSyncTask.setTaskName(taskName);
        tabHaobanSyncTask.setTaskDesc(desc);
        tabHaobanSyncTask.setWxEnterpriseId(wxEnterpriseId);
        tabHaobanSyncTask.setStatusFlag(SyncTaskStatusEnum.init.getVal());
        tabHaobanSyncTask.setTaskId(ToolUtil.randomUUID());
        syncTaskService.createTask(tabHaobanSyncTask);
        lockTask(wxEnterpriseId, tabHaobanSyncTask.getTaskId());
        return tabHaobanSyncTask.getTaskId();

    }


    /**
     * 处理修改数据
     *
     * @param editList
     */
    private void dealEidtSync(List<SyncSingleDealDTO> editList) {
        if (CollectionUtils.isEmpty(editList)) {
            return;
        }
        //先处理修改的
        editList.forEach(syncSingleDealDTO -> {
            // TODO: 2020/4/9  需要优化
            TabHaobanDepartment department = departmentService.selectById(syncSingleDealDTO.getDepartmentId());
            if (!department.getParentDepartmentId().equals(syncSingleDealDTO.getParentDepartmentId())) {
                department.setParentDepartmentId(syncSingleDealDTO.getParentDepartmentId());
                departmentApiService.edit(EntityUtil.changeEntityByJSON(DepartmentDTO.class, department));
            }

        });
    }

    /**
     * 执行新增同步
     *
     * @param taskId
     * @param wxEnterpriseId
     * @param addList
     */
    private void dealAddSync(String taskId, String wxEnterpriseId, List<SyncSingleDealDTO> addList) {

        Map<String, String> pMap = addList.stream().collect(Collectors.toMap(SyncSingleDealDTO::getRelatedId, SyncSingleDealDTO::getParentDepartmentId));
        //获取企业与分组map
        Map<String, List<String>> eidListMap = addList.stream().filter(dto -> {
            String departmentId = dto.getDepartmentId();
            TabHaobanDepartment department = departmentService.selectById(departmentId);
            if (department != null) {
                return false;
            }
            Integer storeFlag = dto.getStoreFlag();
            if (storeFlag.equals(1)) {
                return false;
            }
            return true;
        }).collect(Collectors.groupingBy(SyncSingleDealDTO::getEnterpriseId
                , Collectors.mapping(SyncSingleDealDTO::getRelatedId, Collectors.toList())));

        //放入预处理表
        eidListMap.forEach((k, v) -> {
            //获取所有分组列表
            List<StoreGroupDTO> groupDTOList = storeGroupService.listStoreGroupAndChildren(v.toArray(new String[]{}), k);
            List<TabHaobanPreDealLog> dealList = groupDTOList.stream().map(storeGroupDTO -> {
                TabHaobanPreDealLog dealLog = new TabHaobanPreDealLog();
                dealLog.setDataId(storeGroupDTO.getStoreGroupId());
                if (pMap.containsKey(storeGroupDTO.getStoreGroupId())) {
                    dealLog.setpDataId(pMap.get(storeGroupDTO.getStoreGroupId()));
                } else {
                    dealLog.setpDataId(storeGroupDTO.getParentGroupId());
                }
                dealLog.setDataType(PreDealTypeEnum.group.getVal());
                dealLog.setStatusFlag(0);
                dealLog.setTaskId(taskId);
                dealLog.setEnterpriseId(storeGroupDTO.getEnterpriseId());
                dealLog.setWxEnterpriseId(wxEnterpriseId);
                return dealLog;
            }).collect(Collectors.toList());

            preDealService.insert(dealList);
        });

        //处理门店信息
        List<TabHaobanPreDealLog> storeList = addList.stream().filter(dto -> {
            return dto.getStoreFlag() == 1;
        }).map(dto -> {
            TabHaobanPreDealLog dealLog = new TabHaobanPreDealLog();
            dealLog.setDataId(dto.getRelatedId());
            dealLog.setpDataId(dto.getParentDepartmentId());
            dealLog.setDataType(1);
            dealLog.setStatusFlag(0);
            dealLog.setEnterpriseId(dto.getEnterpriseId());
            dealLog.setTaskId(taskId);
            dealLog.setWxEnterpriseId(wxEnterpriseId);
            return dealLog;
        }).collect(Collectors.toList());

        preDealService.insert(storeList);
    }

    /**
     * 初始化部门
     *
     * @param taskId
     * @param wxEnterpriseId
     * @param addList
     */
    private void dealDepartmentInit(String taskId, String wxEnterpriseId, List<SyncSingleDealDTO> addList) {
        Set<String> addListMid = addList.stream().filter(dto -> dto.getStoreFlag().equals(0)).map(dto -> dto.getRelatedId()).collect(Collectors.toSet());
        if (CollectionUtils.isNotEmpty(addListMid)) {
            dealDepartmentToMq(taskId, addListMid, SyncTaskStatusEnum.group_sync);
        } else {
            addListMid = addList.stream().filter(dto -> dto.getStoreFlag().equals(1)).map(dto -> dto.getRelatedId()).collect(Collectors.toSet());
            dealDepartmentToMq(taskId, addListMid, SyncTaskStatusEnum.store_sync);
        }
    }

    /**
     * 放入mq处理部门数据
     *
     * @param taskId
     * @param dealList
     */
    private void dealDepartmentToMq(String taskId, Set<String> dealList, SyncTaskStatusEnum syncTaskStatusEnum) {
        //预处理分组任务
        syncTaskService.updateTaskStatus(taskId, syncTaskStatusEnum.getVal());
        dealList.forEach(relationId -> {
            DealParamMqDTO dealParamMqDTO = new DealParamMqDTO();
            dealParamMqDTO.setTaskId(taskId);
            dealParamMqDTO.setData(relationId);
            dealParamMqDTO.setType(syncTaskStatusEnum.getVal());
            GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
            try {
                clientInstance.sendMessage("departmentSyncDealMq", JSONObject.toJSONString(dealParamMqDTO));
            } catch (Exception e) {
                logger.info("发送失败:{},{}", taskId, relationId);
                e.printStackTrace();
            }
        });
    }


    public String lockTask(String wxEnterpriseId, String taskId) {
        String key = LOCK_KEY + wxEnterpriseId;
        Object cache = RedisUtil.getCache(key);
        if (null == cache) {
            RedisUtil.setCache(key, taskId, 5L, TimeUnit.DAYS);
            return taskId;
        }
        return null;
    }

    @Override
    public boolean closeTask(String wxEnterpriseId) {
        String taskId = getTaskLock(wxEnterpriseId);
        if (taskId != null) {
            syncTaskService.updateTaskStatus(taskId, SyncTaskStatusEnum.exception_close.getVal());
            unlockTask(wxEnterpriseId);
        }
        return true;
    }

    @Override
    public void unlockTask(String wxEnterpriseId) {
        String key = LOCK_KEY + wxEnterpriseId;
        String taskId = (String) RedisUtil.getCache(key);
        if (StringUtils.isNotBlank(taskId)) {
            TabHaobanSyncTask syncTask = syncTaskService.getSyncTask(taskId);
            if (syncTask.getStatusFlag().equals(SyncTaskStatusEnum.compute.getVal())) {
                String sKey = LOCK_SUCCESS_KEY + wxEnterpriseId;
                RedisUtil.setCache(sKey, taskId);
            }
        }
        RedisUtil.delCache(key);
    }

    @Override
    public String getTaskLock(String wxEnterpriseId) {
        String key = LOCK_KEY + wxEnterpriseId;
        return (String) RedisUtil.getCache(key);
    }

    @Override
    public SyncCheckDTO getTaskCheck(String wxEnterpriseId) {
        String taskId = getTaskLock(wxEnterpriseId);
        SyncCheckDTO syncCheckDTO = new SyncCheckDTO();
        String sKey = LOCK_SUCCESS_KEY + wxEnterpriseId;
        if (null == taskId) {
            taskId = (String) RedisUtil.getCache(sKey);
            RedisUtil.delCache(sKey);
        }
        if (null == taskId) {
            return syncCheckDTO;
        }
        TabHaobanSyncTask syncTask = syncTaskService.getSyncTask(taskId);
        syncCheckDTO.setSyncStatus(syncTask.getStatusFlag());

        int totalCount = preDealService.countByTaskId(taskId, -1, -1);
        int groupErrCount = preDealService.countByTaskId(taskId, PreDealTypeEnum.group.getVal(), PreDealStatusEnum.exception.getVal());
        int storeErrCount = preDealService.countByTaskId(taskId, PreDealTypeEnum.store.getVal(), PreDealStatusEnum.exception.getVal());
        int clerkErrCount = preDealService.countByTaskId(taskId, PreDealTypeEnum.clerk.getVal(), PreDealStatusEnum.exception.getVal());
        int preCount = preDealService.countByTaskId(taskId, -1, PreDealStatusEnum.pre.getVal());


        syncCheckDTO.setTotalDeal(totalCount);
        syncCheckDTO.setGroupErrCount(groupErrCount);
        syncCheckDTO.setStoreErrCount(storeErrCount);
        syncCheckDTO.setStaffErrCount(clerkErrCount);
        syncCheckDTO.setHasDeal(totalCount - preCount);
        return syncCheckDTO;
    }
}
