package com.gic.haoban.manage.service.service.out.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import cn.hutool.core.collection.CollectionUtil;

import com.gic.haoban.base.api.common.Constant;
import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.manage.api.dto.StaffDepartmentRelatedDTO;
import com.gic.haoban.manage.api.service.StaffApiService;

import com.gic.haoban.manage.api.service.StaffDepartmentRelatedApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.gic.api.base.commons.JSONResponse;
import com.gic.api.base.commons.Page;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.dto.StoreGroupDTO;
import com.gic.enterprise.api.service.StoreGroupService;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.common.utils.StringUtil;
import com.gic.haoban.contacts.manage.api.dto.SyncJsonColumnDTO;
import com.gic.haoban.manage.api.dto.DepartmentDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.service.DepartmentApiService;
import com.gic.haoban.manage.service.dao.mapper.DepartmentMapper;
import com.gic.haoban.manage.service.entity.TabHaobanDepartment;
import com.gic.haoban.manage.service.entity.TabHaobanStaffDepartmentRelated;
import com.gic.haoban.manage.service.entity.TabHaobanSyncErrorLog;
import com.gic.haoban.manage.service.entity.TabHaobanWxEnterpriseRelated;
import com.gic.haoban.manage.service.service.DepartmentService;
import com.gic.haoban.manage.service.service.StaffDepartmentRelatedService;
import com.gic.haoban.manage.service.service.SyncErrorLogService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.redis.data.util.RedisUtil;
import com.gic.wechat.api.service.qywx.QywxDepartmentApiService;
import com.github.pagehelper.PageHelper;

@Service
public class DepartmentApiServiceImpl implements DepartmentApiService {
	
	private static Logger logger  = LoggerFactory.getLogger(DepartmentApiServiceImpl.class);
	@Autowired
	private DepartmentService departmentService;
	@Autowired
	private StaffDepartmentRelatedService staffDepartmentService;
	@Autowired
	private DepartmentMapper departmentMapper;
	@Autowired
	private com.gic.enterprise.api.service.DepartmentService gicDepartmentService;
	@Autowired
	private QywxDepartmentApiService qywxDepartmentApiService;
	@Autowired
	private WxEnterpriseService wxEnterpriseService;
	@Autowired
	private StoreService storeService;
	@Autowired
	private StoreGroupService storeGroupService;
	@Autowired
	private StaffApiService staffApiService;

	@Autowired
	private StaffDepartmentRelatedApiService staffDepartmentRelatedApiService;
	@Autowired
	private SyncErrorLogService syncErrorLogService;
	
	
	@Override
	public List<DepartmentDTO> listByParentId(String parentId) {
		List<TabHaobanDepartment> list = departmentService.listByParentId(parentId);
		
		return EntityUtil.changeEntityListByJSON(DepartmentDTO.class, list);
	}

	@Override
	public DepartmentDTO selectById(String departmentId) {
		TabHaobanDepartment tab = departmentService.selectById(departmentId);
		return EntityUtil.changeEntityByJSON(DepartmentDTO.class, tab);
	}

	@Override
	public ServiceResponse<DepartmentDTO> add(DepartmentDTO department) {
		ServiceResponse hr = new ServiceResponse();
		hr.setCode(1);
		//调微信的新增接口
		String wxDepartmentId = "";
		WxEnterpriseDTO enterpriseDTO = wxEnterpriseService.selectById(department.getWxEnterpriseId());
		logger.info("【部门新增】enterpriseDTO = {}",JSON.toJSONString(enterpriseDTO));
		if(enterpriseDTO != null) {
			Integer sort = department.getSort();
			if(sort == null) {
				int maxSort = departmentMapper.selectMaxSort(department.getParentDepartmentId());
				department.setSort(maxSort + 1);
			}
			
			com.gic.wechat.api.dto.qywx.DepartmentDTO dto = new com.gic.wechat.api.dto.qywx.DepartmentDTO();
			dto.setCorpid(enterpriseDTO.getCorpid());
			dto.setName(department.getDepartmentName());
			dto.setOrder(department.getSort());
			TabHaobanDepartment parent = departmentService.selectById(department.getParentDepartmentId());
			dto.setParentid(Integer.valueOf(parent.getWxDepartmentId()));
			logger.info("【部门新增】corpid() = {},siteId={},dto={}",enterpriseDTO.getCorpid(),enterpriseDTO.getContactSecret(),JSON.toJSONString(dto));
			JSONResponse jp = qywxDepartmentApiService.createSelfDepartment(enterpriseDTO.getCorpid(), enterpriseDTO.getContactSecret(), dto);
			logger.info("【部门新增】jp = {}",JSON.toJSONString(jp));
			if(jp.getErrorCode() != 0) {
				hr.setCode(0);
				hr.setMessage("微信新增部门失败");
				return hr;
			}
			wxDepartmentId = jp.getResult().toString();
			logger.info("【部门新增】wxDepartmentId = {}",wxDepartmentId);
			department.setWxDepartmentId(wxDepartmentId);
			department.setLevel(parent.getLevel() + 1);
			setGicEnterpriseId(department);
			String departmentId = departmentService.add(department);
			department.setDepartmentId(departmentId);
			hr.setResult(department);
		}
		
		return hr;
	}

	private void setGicEnterpriseId(DepartmentDTO department) {
		String relatedId = department.getRelatedId();
		if(StringUtils.isNotBlank(relatedId)){
			Integer storeFlag = department.getIsStore();
			if(storeFlag == 1){
				StoreDTO store  = storeService.getStore(relatedId);
				if(store != null){
					department.setEnterpriseId(store.getEnterpriseId());
				}
			}else{
				com.gic.enterprise.api.dto.DepartmentDTO departmentDTO = gicDepartmentService.getDeptment(relatedId);
				if(departmentDTO != null){
					department.setEnterpriseId(departmentDTO.getEnterpriseId());
				}
				StoreGroupDTO storeGroup = storeGroupService.getStoreGroupById(relatedId);
				if(storeGroup != null){
					department.setEnterpriseId(storeGroup.getEnterpriseId());
				}
			}
		}
	}

	@Override
	public ServiceResponse edit(DepartmentDTO department) {
		ServiceResponse hr = new ServiceResponse();
		hr.setCode(1);
		hr.setResult(department);
		//调微信的修改接口
		TabHaobanDepartment tab = departmentService.selectById(department.getDepartmentId());
		if(tab == null){
			hr.setCode(0);
			hr.setMessage("部门不存在");
			return hr;
		}
		logger.info("【部门修改】tab={}",JSON.toJSONString(tab));
		WxEnterpriseDTO enterpriseDTO = wxEnterpriseService.selectById(tab.getWxEnterpriseId());
		if(enterpriseDTO != null) {
			com.gic.wechat.api.dto.qywx.DepartmentDTO dto = new com.gic.wechat.api.dto.qywx.DepartmentDTO();
			dto.setCorpid(enterpriseDTO.getCorpid());
			dto.setName(department.getDepartmentName());
			dto.setOrder(department.getSort());
			TabHaobanDepartment parent = departmentService.selectById(department.getParentDepartmentId());
			dto.setParentid(Integer.valueOf(parent.getWxDepartmentId()));
			dto.setId(Integer.valueOf(tab.getWxDepartmentId()));
			JSONResponse jp = qywxDepartmentApiService.updateSelfDepartment(enterpriseDTO.getCorpid(), enterpriseDTO.getContactSecret(), dto);
			logger.info("【部门修改】jp={}",JSON.toJSONString(jp));
			if(jp.getErrorCode() != 0) {
				hr.setCode(0);
				hr.setMessage("微信修改部门失败");
				return hr;
			}
		}
		
		departmentService.edit(department);
		return hr;
	}

	@Override
	public ServiceResponse del(String departmentId) {
		ServiceResponse hr = new ServiceResponse();
		hr.setCode(1);
		
		List<TabHaobanDepartment> list = departmentService.listByParentId(departmentId);
		logger.info("【部门删除】list.size={}",list.size());
		if(!list.isEmpty()) {
			hr.setCode(0);
			hr.setMessage("存在子部门，删除部门失败");
			return hr;
		}
		
		List<TabHaobanStaffDepartmentRelated> staffList = staffDepartmentService.listByDepartmentId(departmentId);
		logger.info("【部门删除】staffList.size={}",staffList.size());
		if(!staffList.isEmpty()) {
			hr.setCode(0);
			hr.setMessage("存在成员，删除部门失败");
			return hr;
		}
		//调微信的删除接口
		TabHaobanDepartment department = departmentService.selectById(departmentId);
		if(department == null){
			hr.setCode(0);
			hr.setMessage("不存在部门，删除部门失败");
			return hr;
		}
		WxEnterpriseDTO enterpriseDTO = wxEnterpriseService.selectById(department.getWxEnterpriseId());
		if(enterpriseDTO != null) {
			JSONResponse jp = qywxDepartmentApiService.deleteSelfDepartment(enterpriseDTO.getCorpid(), enterpriseDTO.getContactSecret(), Integer.parseInt(department.getWxDepartmentId()));
			if(jp.getErrorCode() != 0) {
				hr.setCode(0);
				hr.setMessage("微信删除部门失败");
				return hr;
			}
		}
		
		departmentService.del(departmentId);
		return hr;
	}
	
	@Override
	public ServiceResponse wxCallBackDel(String departmentId){
		ServiceResponse hr = new ServiceResponse();
		hr.setCode(1);
		
		List<TabHaobanDepartment> list = departmentService.listByParentId(departmentId);
		logger.info("【部门删除】list.size={}",list.size());
		if(!list.isEmpty()) {
			hr.setCode(0);
			hr.setMessage("存在子部门，删除部门失败");
			return hr;
		}
		
		List<TabHaobanStaffDepartmentRelated> staffList = staffDepartmentService.listByDepartmentId(departmentId);
		logger.info("【部门删除】staffList.size={}",staffList.size());
		if(!staffList.isEmpty()) {
			hr.setCode(0);
			hr.setMessage("存在成员，删除部门失败");
			return hr;
		}
		departmentService.del(departmentId);
		return hr;
	}

	@Override
	public DepartmentDTO selectByRelatedId(String relatedId) {
		TabHaobanDepartment tab = departmentService.selectByRelatedId(relatedId);
		return EntityUtil.changeEntityByJSON(DepartmentDTO.class, tab);
	}

	@Override
	public ServiceResponse recycle(String departmentId) {
		ServiceResponse hr = new ServiceResponse();
		hr.setCode(1);
		
		List<TabHaobanDepartment> list = departmentService.listByParentId(departmentId);
		logger.info("【部门删除】list.size={}",list.size());
		if(!list.isEmpty()) {
			hr.setCode(0);
			hr.setMessage("存在子部门，删除部门失败");
			return hr;
		}
		
		List<TabHaobanStaffDepartmentRelated> staffList = staffDepartmentService.listByDepartmentId(departmentId);
		logger.info("【部门删除】staffList.size={}",staffList.size());
		if(!staffList.isEmpty()) {
			hr.setCode(0);
			hr.setMessage("存在成员，删除部门失败");
			return hr;
		}
		//调微信的新增接口
		TabHaobanDepartment department = departmentService.selectById(departmentId);
		WxEnterpriseDTO enterpriseDTO = wxEnterpriseService.selectById(department.getWxEnterpriseId());
		if(enterpriseDTO != null) {
			JSONResponse jp = qywxDepartmentApiService.deleteSelfDepartment(enterpriseDTO.getCorpid(), enterpriseDTO.getContactSecret(), Integer.parseInt(department.getWxDepartmentId()));
			if(jp.getErrorCode() != 0) {
				hr.setCode(0);
				hr.setMessage("微信删除部门失败");
				return hr;
			}
		}
		
		departmentService.recycle(departmentId);
		return hr;
	}
	
	@Override
	public ServiceResponse wxCallBackrecycle(String departmentId) {
		ServiceResponse hr = new ServiceResponse();
		hr.setCode(1);
		
		List<TabHaobanDepartment> list = departmentService.listByParentId(departmentId);
		logger.info("【部门删除】list.size={}",list.size());
		if(!list.isEmpty()) {
			hr.setCode(0);
			hr.setMessage("存在子部门，删除部门失败");
			return hr;
		}
		
		List<TabHaobanStaffDepartmentRelated> staffList = staffDepartmentService.listByDepartmentId(departmentId);
		logger.info("【部门删除】staffList.size={}",staffList.size());
		if(!staffList.isEmpty()) {
			hr.setCode(0);
			hr.setMessage("存在成员，删除部门失败");
			return hr;
		}
		
		departmentService.recycle(departmentId);
		return hr;
	}

	@Override
	public Page<DepartmentDTO> pageDepartmentByParams(BasePageInfo pageInfo, String wxEnterpriseId, String keyword,
			Integer storeFlag, Integer recycleFlag) {
		PageHelper.startPage(pageInfo.getPageNum(), pageInfo.getPageSize());
		return  PageUtil.changePageHelperToCurrentPage(departmentService.pageDepartmentByParams(wxEnterpriseId,keyword,storeFlag,recycleFlag),DepartmentDTO.class);
	}

	@Override
	public ServiceResponse repairRecycle(String departmentId,String parentId) {
		ServiceResponse hr = new ServiceResponse();
		hr.setCode(0);
		TabHaobanDepartment tab = departmentService.selectById(departmentId);
		//调微信的新增接口
		String wxDepartmentId = "";
		WxEnterpriseDTO enterpriseDTO = wxEnterpriseService.selectById(tab.getWxEnterpriseId());
		logger.info("【部门新增】enterpriseDTO = {}",JSON.toJSONString(enterpriseDTO));
		if(enterpriseDTO != null) {
			com.gic.wechat.api.dto.qywx.DepartmentDTO dto = new com.gic.wechat.api.dto.qywx.DepartmentDTO();
			dto.setCorpid(enterpriseDTO.getCorpid());
			dto.setName(tab.getDepartmentName());
			dto.setOrder(tab.getSort());
			TabHaobanDepartment parent = departmentService.selectById(parentId);
			dto.setParentid(Integer.valueOf(parent.getWxDepartmentId()));
			logger.info("【部门新增】corpid() = {},siteId={},dto={}",enterpriseDTO.getCorpid(),enterpriseDTO.getContactSecret(),JSON.toJSONString(dto));
			JSONResponse jp = qywxDepartmentApiService.createSelfDepartment(enterpriseDTO.getCorpid(), enterpriseDTO.getContactSecret(), dto);
			logger.info("【部门新增】jp = {}",JSON.toJSONString(jp));
			if(jp.getErrorCode() != 0) {
				hr.setCode(0);
				hr.setMessage("微信新增部门失败");
				return hr;
			}
			wxDepartmentId = jp.getResult().toString();
			tab.setWxDepartmentId(wxDepartmentId);
			tab.setParentDepartmentId(parentId);
			logger.info("【部门新增】wxDepartmentId = {}",wxDepartmentId);
		}
		departmentService.repairRecycle(tab);
		
		return hr;
		
	}

	@Override
	public List<DepartmentDTO> listByDepartmentIds(List<String> departmentIds,
			int storeFlag) {
		List<TabHaobanDepartment> list= departmentMapper.listByDepartmentIds(departmentIds, storeFlag);
		if(list==null){
			return new ArrayList<DepartmentDTO>();
		}
		return EntityUtil.changeEntityListByJSON(DepartmentDTO.class, list);
	}

	@Override
	public List<DepartmentDTO> listByRelatedIds(List<String> relatedIds) {
		List<TabHaobanDepartment> list= departmentMapper.listByRelatedIds(relatedIds);
		if(list==null){
			return new ArrayList<DepartmentDTO>();
		}
		return EntityUtil.changeEntityListByJSON(DepartmentDTO.class, list);
	}
	@Override
	public DepartmentDTO getRootByEnterpriseId(String wxEnterpriseId) {
		
		return departmentService.getRootByEnterpriseId(wxEnterpriseId);
	}

	@Override
	public List<DepartmentDTO> listStoreByWxEnterpriseId(String wxEnterpriseId) {
		List<TabHaobanDepartment> list = departmentMapper.listStoreByWxEnterpriseId(wxEnterpriseId);
		return EntityUtil.changeEntityListByJSON(DepartmentDTO.class, list);
	}

	@Override
	public Page<DepartmentDTO> pageFullStoreByWxEnterpriseId(
			String wxEnterpriseId,String search,List<String> storeIds, int count,BasePageInfo pageInfo) {
		PageHelper.startPage(pageInfo.getPageNum(), pageInfo.getPageSize());
		if(CollectionUtil.isNotEmpty(storeIds)&&StringUtils.isNotEmpty(search)){
			return PageUtil.changePageHelperToCurrentPage( departmentMapper.pageFullStoreByWxEnterpriseId2(wxEnterpriseId,search, storeIds,count),DepartmentDTO.class);
		}
		return PageUtil.changePageHelperToCurrentPage( departmentMapper.pageFullStoreByWxEnterpriseId(wxEnterpriseId,search, storeIds,count),DepartmentDTO.class);
	}

	@Override
	public void syncGicStore(String fieldListString) {
		logger.info("【同步门店】fieldList={}",fieldListString);
		List<SyncJsonColumnDTO> fieldList = JSONArray.parseArray(fieldListString,SyncJsonColumnDTO.class);
		if(fieldList.isEmpty()){
	        return;
	    }
		
		Integer status = 0;
		String storeName = "";
		String departmentId = "";
		String storeId = "";
		for (SyncJsonColumnDTO syncJsonColumnDTO:fieldList){
	    	String name = StringUtil.camelName(syncJsonColumnDTO.getName());
	        String value = syncJsonColumnDTO.getValue();
	        if("status".equals(name)) {
	        	status = Integer.parseInt(value);
	        }
	        if("storeName".equals(name)) {
	        	storeName = value;
	        }
	        if("departmentId".equals(name)) {
	        	departmentId = value;
	        }
	        if("storeId".equals(name)) {
	        	storeId = value;
	        }
		}
		DepartmentDTO department = departmentService.getByRelatedId(storeId);
		if(department == null) {
			if(status != null && status == 2) {
				DepartmentDTO parent = getParent(departmentId);
				if(parent == null) {
					TabHaobanWxEnterpriseRelated tab = null;
					//parent = getRootByEnterpriseId(wxEnterpriseId)；
				}
			}
		}
		
	        
		
	}
	/**
	 * 向上找父节点
	 * @param departmentId
	 * @return
	 */
	private DepartmentDTO getParent(String departmentId) {
		DepartmentDTO department = departmentService.getByRelatedId(departmentId);
		if(department == null) {
			com.gic.enterprise.api.dto.DepartmentDTO dto = gicDepartmentService.getDeptment(departmentId);
			String parentId = dto.getParentDepartId();
			getParent(parentId);
		}
		return department;
	}

	@Override
	public void syncGicDepartment(String fieldListString) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public Page<DepartmentDTO> pageByParentId(String parentId,String keyword, BasePageInfo pageInfo) {
		PageHelper.startPage(pageInfo.getPageNum(), pageInfo.getPageSize());
		return  PageUtil.changePageHelperToCurrentPage(departmentService.pageByParentId(parentId,keyword),DepartmentDTO.class);
	}

	@Override
	public List<DepartmentDTO> listByChainId(String chainId,String wxEnterpriseId) {
		List<TabHaobanDepartment> list = departmentService.listByChainId(chainId,wxEnterpriseId);
		return EntityUtil.changeEntityListByJSON(DepartmentDTO.class, list);
	}

	@Override
	public void initwxDepartment(String corpid, String contactSecret, String wxEnterpriseId) {
//		List<com.gic.wechat.api.dto.qywx.DepartmentDTO> list = this.qywxDepartmentApiService.listDepartment(corpid, suiteid, null);
//		logger.info("企业所有部门数据：{}", JSON.toJSONString(list));
//		
//		com.gic.wechat.api.dto.qywx.DepartmentDTO rootDepartment = null;
//		for (com.gic.wechat.api.dto.qywx.DepartmentDTO departmentDTO : list) {
//			if("0".equals(departmentDTO.getParentid())){
//				rootDepartment = departmentDTO;
//				break;
//			}
//		}
//		
//		String parentId = addDepartment(rootDepartment, wxEnterpriseId, null,corpid,suiteid);
//		List<com.gic.wechat.api.dto.qywx.DepartmentDTO> subList = new ArrayList<>();
//		for (com.gic.wechat.api.dto.qywx.DepartmentDTO departmentDTO : list) {
//			if(rootDepartment.getId().equals(departmentDTO.getParentid())){
//				subList.add(departmentDTO);
//			}
//		}
//		
//		DepartmentDTO parent = EntityUtil.changeEntityByJSON(DepartmentDTO.class,departmentService.selectById(parentId));
//		this.addSon(subList, wxEnterpriseId, parent,corpid,suiteid);
		RedisUtil.lock("init_enterprise_" + wxEnterpriseId,60 * 60L);
		String taskId = StringUtil.randomUUID();
		List<com.gic.wechat.api.dto.qywx.DepartmentDTO> list = this.qywxDepartmentApiService.listSelfDepartment(corpid, contactSecret, null);
		logger.info("企业所有部门数据：{}", JSON.toJSONString(list));
		if(CollectionUtils.isNotEmpty(list)){
			this.addAllDepartment(list, 0, wxEnterpriseId, corpid, contactSecret,taskId);
		}
		RedisUtil.unlock("init_enterprise_" + wxEnterpriseId);
	}
	
	@Override
	public boolean isInitLocked(String wxEnterpriseId){
		return RedisUtil.isLocked("init_enterprise_" + wxEnterpriseId);
		
	}
	


	private void addAllDepartment(List<com.gic.wechat.api.dto.qywx.DepartmentDTO> list, Integer parentId,
								  String wxEnterpriseId, String corpid, String contactSecret,String taskId){
		int level = 1;
		String chainId = "0";
		String chainName = "0";
		String pId = "0";
		if(parentId != 0){
			TabHaobanDepartment parent = this.departmentService.getByWxId(parentId + "", wxEnterpriseId);
			if(parent != null){
				level = parent.getLevel() + 1;
				chainId = parent.getChainId() + Constant.ID_SEPARATOR + parent.getDepartmentId();
				chainName = parent.getChainName() + Constant.NAME_SEPARATOR + parent.getDepartmentName();
				pId = parent.getDepartmentId();
			}
		}
		for(com.gic.wechat.api.dto.qywx.DepartmentDTO departmentDTO : list){
			try {
				if (parentId.equals(departmentDTO.getParentid())) {
					TabHaobanDepartment exist = this.departmentService.getByWxId(departmentDTO.getId() + "", wxEnterpriseId);
					logger.info("分组是否存在：{}", JSON.toJSONString(exist));
					if(exist == null){
						com.gic.haoban.manage.api.dto.DepartmentDTO dto = new com.gic.haoban.manage.api.dto.DepartmentDTO();
						dto.setDepartmentName(departmentDTO.getName());
						dto.setLevel(level);
						dto.setWxEnterpriseId(wxEnterpriseId);
						dto.setWxDepartmentId(departmentDTO.getId()+"");
						dto.setParentDepartmentId(pId);
						dto.setChainId(chainId);
						dto.setChainName(chainName);
						dto.setSort(departmentDTO.getOrder());
						dto.setIsStore(0);
						this.departmentService.add(dto);
					}
				}
			} catch (Exception e) {
				TabHaobanSyncErrorLog tab = new TabHaobanSyncErrorLog();
				tab.setDataId(departmentDTO.getId()+"");
				tab.setDataType(0);
				tab.setLogType(0);
				tab.setReason(e.getMessage());
				tab.setTaskId(taskId);
				tab.setWxEnterpriseId(wxEnterpriseId);
				syncErrorLogService.add(tab);
			}
			this.addAllDepartment(list, departmentDTO.getId(), wxEnterpriseId, corpid, contactSecret,taskId);
			this.staffApiService.initWxUser(corpid, contactSecret, wxEnterpriseId, departmentDTO.getId(),taskId);
		}
	}

	@Override
	public Integer departmentRecycleCount(String wxEnterpriseId) {
		return departmentService.departmentRecycleCount(wxEnterpriseId);
	}

	@Override
	public boolean hasSonNode(String departmentId) {
		int departmentCount = departmentService.countByDepartment(departmentId);
		if(departmentCount > 0){
			return true;
		}
		return false;
	}

	@Override
	public ServiceResponse gicDel(String departmentId) {
		ServiceResponse hr = new ServiceResponse();
		hr.setCode(1);
		
		List<TabHaobanDepartment> list = departmentService.listByParentId(departmentId);
		logger.info("【部门删除】list.size={}",list.size());
		if(!list.isEmpty()) {
			hr.setCode(0);
			hr.setMessage("存在子部门，删除部门失败");
			//return hr;
			TabHaobanDepartment department = departmentService.selectById(departmentId);
			if(department != null){
				department.setGicDelFlag(1);
			}
			departmentMapper.updateByPrimaryKeySelective(department);
		}
		
		List<TabHaobanStaffDepartmentRelated> staffList = staffDepartmentService.listByDepartmentId(departmentId);
		logger.info("【部门删除】staffList.size={}",staffList.size());
		if(!staffList.isEmpty()) {
			hr.setCode(0);
			hr.setMessage("存在成员，删除部门失败");
			//return hr;
			TabHaobanDepartment department = departmentService.selectById(departmentId);
			if(department != null){
				department.setGicDelFlag(1);
			}
			departmentMapper.updateByPrimaryKeySelective(department);
		}
		//调微信的删除接口
		TabHaobanDepartment department = departmentService.selectById(departmentId);
		WxEnterpriseDTO enterpriseDTO = wxEnterpriseService.selectById(department.getWxEnterpriseId());
		if(enterpriseDTO != null) {
			JSONResponse jp = qywxDepartmentApiService.deleteSelfDepartment(enterpriseDTO.getCorpid(), enterpriseDTO.getContactSecret(), Integer.parseInt(department.getWxDepartmentId()));
			if(jp.getErrorCode() != 0) {
				hr.setCode(0);
				hr.setMessage("微信删除部门失败");
				return hr;
			}
		}
		
		departmentService.del(departmentId);
		return hr;
		
	}


	@Override
	public ServiceResponse<List<DepartmentDTO>> listStoreListByStaffId(String staffId) {
		ServiceResponse<List<DepartmentDTO>> rer = new ServiceResponse<List<DepartmentDTO>>();
		List<StaffDepartmentRelatedDTO> relateList = staffDepartmentRelatedApiService.listByStaffId(staffId);
		if (CollectionUtils.isEmpty(relateList)) {
			logger.info("没有部分显示:{}", staffId);
			return rer;
		}
		List<String> departmentIds = relateList.stream().map(dto -> dto.getDepartmentId()).collect(Collectors.toList());
		List<DepartmentDTO> departmentList = listByDepartmentIds(departmentIds, 1);
		rer.setResult(departmentList);
		return rer;
	}
	
	@Override
	public void initwxDepartmentMQ(String res) {
		WxEnterpriseDTO enterpriseDTO = this.wxEnterpriseService.selectById(res);
		if(enterpriseDTO != null){
			if(StringUtils.isNotBlank(enterpriseDTO.getContactSecret()) && enterpriseDTO.getContactFlag() != null && enterpriseDTO.getContactFlag() == 0){
				this.initwxDepartment(enterpriseDTO.getCorpid(), enterpriseDTO.getContactSecret(), enterpriseDTO.getWxEnterpriseId());
				enterpriseDTO.setContactFlag(1);
				this.wxEnterpriseService.update(enterpriseDTO);
			}
		}
		
	}
}
