package com.gic.haoban.manage.service.service.out.impl;

import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.utils.URLEncodedUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import cn.hutool.core.collection.CollectionUtil;

import com.alibaba.csp.sentinel.util.StringUtil;
import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.api.dto.MessageDTO;
import com.gic.haoban.manage.api.dto.ApplicationDTO;
import com.gic.haoban.manage.api.dto.StaffDepartmentRelatedDTO;
import com.gic.haoban.manage.api.enums.AppPageType;
import com.gic.haoban.manage.api.service.ApplicationApiService;
import com.gic.haoban.manage.api.service.StaffDepartmentRelatedApiService;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.dao.mapper.StaffDepartmentRelatedMapper;
import com.gic.haoban.manage.service.entity.TabHaobanApplication;
import com.gic.haoban.manage.service.entity.TabHaobanStaffDepartmentRelated;
import com.gic.haoban.manage.service.service.ApplicationService;
import com.gic.haoban.manage.service.service.StaffDepartmentRelatedService;
import com.gic.wechat.api.dto.qywx.QywxXcxSendMessageDTO;
import com.gic.wechat.api.service.qywx.QywxSuiteApiService;

import org.springframework.stereotype.Service;

@Service	
public class StaffDepartmentRelatedApiServiceImpl implements StaffDepartmentRelatedApiService {
    private static Logger logger= LoggerFactory.getLogger(StaffDepartmentRelatedApiServiceImpl.class);

	@Autowired
	private StaffDepartmentRelatedService staffDepartmentRelatedService;
	@Autowired
	private StaffDepartmentRelatedMapper staffDepartmentRelatedMapper;
	@Autowired
	private ClerkService clerkService;
	@Autowired
	private QywxSuiteApiService qywxSuiteApiService;
	@Autowired
	private Config config;
	
	@Override
	public List<StaffDepartmentRelatedDTO> listByDepartmentId(
			String departmentId) {
		List<TabHaobanStaffDepartmentRelated> list = staffDepartmentRelatedService.listByDepartmentId(departmentId);
		List<StaffDepartmentRelatedDTO> resultList = EntityUtil.changeEntityListByJSON(StaffDepartmentRelatedDTO.class, list);
		return resultList;
	}

	@Override
	public void updateById(StaffDepartmentRelatedDTO dto) {
		TabHaobanStaffDepartmentRelated tab = EntityUtil.changeEntityByJSON(TabHaobanStaffDepartmentRelated.class, dto);
		staffDepartmentRelatedMapper.updateByPrimaryKeySelective(tab);
	}

	@Override
	public void deleteCode(StaffDepartmentRelatedDTO dto) {
		staffDepartmentRelatedMapper.deleteCode(EntityUtil.changeEntityByJSON(TabHaobanStaffDepartmentRelated.class, dto));		
	}

	@Override
	public StaffDepartmentRelatedDTO getByStaffDepartmentRelatedId(String staffDepartmentRelatedId) {
		TabHaobanStaffDepartmentRelated tab = staffDepartmentRelatedMapper.getById(staffDepartmentRelatedId);
		if(Objects.isNull(tab)){
			return null;
		}
		StaffDepartmentRelatedDTO dto = EntityUtil.changeEntity(StaffDepartmentRelatedDTO.class, tab);
		return dto;
	}

	@Override
	public List<StaffDepartmentRelatedDTO> listByStaffId(String staffId) {
		List<TabHaobanStaffDepartmentRelated> list = staffDepartmentRelatedMapper.listStaffDepartmentByStaffId(staffId);
		if(CollectionUtil.isEmpty(list)){
			return new ArrayList<StaffDepartmentRelatedDTO>();
		}
		List<StaffDepartmentRelatedDTO> resultList = EntityUtil.changeEntityListByJSON(StaffDepartmentRelatedDTO.class, list);
		return resultList;
	}

	@Override
	public StaffDepartmentRelatedDTO getOneByClerkCodeAndDepartmentId(String clerkCode, String departmentId) {
		TabHaobanStaffDepartmentRelated tab = staffDepartmentRelatedMapper.getDepartmentIdAndCode(departmentId,clerkCode);
		if(tab== null){
			return null;
		}
		StaffDepartmentRelatedDTO dto = EntityUtil.changeEntity(StaffDepartmentRelatedDTO.class, tab);
		return dto;
	}
	@Override
	public StaffDepartmentRelatedDTO getOneByStaffIdAndDepartmentId(String staffId, String departmentId) {
		TabHaobanStaffDepartmentRelated tab = staffDepartmentRelatedMapper.getDepartmentIdAndStaffId(departmentId,staffId);
		if(tab== null){
			return null;
		}
		StaffDepartmentRelatedDTO dto = EntityUtil.changeEntity(StaffDepartmentRelatedDTO.class, tab);
		return dto;
	}

	@Override
	public String getWxUserIdByClerkId(String clerkId) {
		ClerkDTO clerkDTO = clerkService.getclerkById(clerkId);
		if(clerkDTO == null){
			return null;
		}
		String clerkCode =clerkDTO.getClerkCode();
		if(StringUtil.isEmpty(clerkCode) ){
			return null;
		}
		List<TabHaobanStaffDepartmentRelated> list = staffDepartmentRelatedMapper.listByClerkCode(clerkCode);
		if(CollectionUtil.isEmpty(list)){
			return null;
		}
		String wxUserId = list.get(0).getWxUserId();
		return wxUserId;
	}

	@Override
	public boolean sendSingleMessage(String clerkId, String title,
			String content, String pageUrl) {
		String wxUserId = getWxUserIdByClerkId(clerkId);
		if(StringUtils.isEmpty(wxUserId)){
			logger.info("wxUserId不存在============clerkId={}",clerkId);
			return false;
		}
		QywxXcxSendMessageDTO messageDTO = new QywxXcxSendMessageDTO();
        ArrayList<String> list = new ArrayList<>();
        list.add(wxUserId);
		messageDTO.setAppid(config.getAppid());
        messageDTO.setUserIds(list);
        messageDTO.setPage(pageUrl);
        messageDTO.setTitle(title);
        messageDTO.setDescription(content);
		boolean b = qywxSuiteApiService.sendMessage(config.getCorpid(), config.getSuiteId(), messageDTO);
		return b;
	}

	@Override
	public String getPageUrl(int type, String data) {
		String baseUrl = "/pages/route/index";
		String addurl = AppPageType.getDescByCode(type);
		String url = "";
		if(type ==  AppPageType.INDEX.getCode()){
			//工作台
			url = baseUrl+"?pageType="+addurl;
		}else{
			url = baseUrl+"?pageType="+addurl;
		}
		url = url+"&data="+data;
		String resultUrl = URLEncoder.encode(url);
		return resultUrl;
	}
	

}
